﻿/*	File			KnownPeers.cpp
	Purpose			Library class demonstration
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			February 2015
*/

using System;
using System.Net;
using System.Runtime.InteropServices;

namespace DemonstrationLibrary_for_MediaShare_Peer
{
    public class KnownPeers
    {
        public const Int32 MAX_KNOWN_PEERS = 10;

        public struct Known_Peer_details 
        {
	        public Int32 iPeer_ID;
            public IPEndPoint Peer_IPEndPoint;
            public bool bInUse;
	        public bool bMediaDataReceived;
        };

	    private Known_Peer_details[] m_Known_Peers_List;

        public KnownPeers()
        {
	        Initialise_KnownPeers_Array();
        }
 
        private void Initialise_KnownPeers_Array()
        {
            m_Known_Peers_List = new Known_Peer_details[MAX_KNOWN_PEERS];

            for (int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
	        {
		        m_Known_Peers_List[iIndex].bInUse = false;		
		        m_Known_Peers_List[iIndex].bMediaDataReceived = false;
	        }
        }

        private Int32 GetIndexOf_Available_KnownPeersList_Entry()
        { 
	        int iNextAvailableEntryIndex = -1;	// Signals that the array is full
	        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
	        {
		        if(false == m_Known_Peers_List[iIndex].bInUse)
		        {
			        iNextAvailableEntryIndex = iIndex;
			        break;
		        }
	        }
	        return iNextAvailableEntryIndex;
        }

        public Int32 GetIndexOf_KnownPeer_InPeersList(int iPeerID)
        {	
	        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
	        {
		        if(true == m_Known_Peers_List[iIndex].bInUse)
		        {
			        if(iPeerID == m_Known_Peers_List[iIndex].iPeer_ID)
			        {  // Matched by Peer ID
				        return iIndex;
			        }
		        }
	        }
	        return -1;
        }

        public bool CheckIfPeerExistsIn_KnownPeersList(int iPeerID, IPEndPoint Peer_IPEndPoint)
        { 
	        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
	        {
		        if(true == m_Known_Peers_List[iIndex].bInUse)
		        {
			        if(iPeerID == m_Known_Peers_List[iIndex].iPeer_ID &&
                        m_Known_Peers_List[iIndex].Peer_IPEndPoint.ToString().Equals(Peer_IPEndPoint.ToString()))
			        {  // Matched by combination of ID and IP address
				        return true;
			        }
		        }
	        }
	        return false;
        }

        public bool Add_NewlyDiscoveredPeerTo_KnownPeersList(int iPeer_ID, IPEndPoint Peer_IPEndPoint)
        {
	        int iIndex = GetIndexOf_Available_KnownPeersList_Entry();
	        if(-1 == iIndex)
	        {
		        return false;	// No space in array
	        }		
	        m_Known_Peers_List[iIndex].bInUse = true;
	        m_Known_Peers_List[iIndex].iPeer_ID = iPeer_ID;
            m_Known_Peers_List[iIndex].Peer_IPEndPoint = Peer_IPEndPoint;
	        return true;
        }
	
        public Known_Peer_details get_Known_Peers_List_entry_byIndex(int iIndex)
        {
	        if(iIndex >= 0 && iIndex < MAX_KNOWN_PEERS)
	        {
		        return m_Known_Peers_List[iIndex];
	        }

	        // Initialise an empty structure to return if parameter is out of range
	        Known_Peer_details empty_struct = new Known_Peer_details(); 
	        empty_struct.bInUse = false;
	        empty_struct.bMediaDataReceived = false;
	        empty_struct.iPeer_ID = -1;
            empty_struct.Peer_IPEndPoint = new IPEndPoint(IPAddress.Loopback, 0);
	        return empty_struct;
        }	

        public void set_Known_Peers_List_entry_bMediaDataReceived_flag_byIndex(int iIndex)
        {
	        if(iIndex >= 0 && iIndex < MAX_KNOWN_PEERS)
	        {
		        m_Known_Peers_List[iIndex].bMediaDataReceived = true;
	        }
        }   
    }
}
